<!DOCTYPE html>
<html lang="en">
  <head>
    {% include "base/head.html" %}
    {% block addl_head %}{# Custom style sheets or other head includes #}
    {% endblock addl_head %}
  </head>
  <body>
  {% import "base/macros.html" as macros %}
  {% import "base/analytics_macros.html" as analytics_macros %}
  <header>
    {%- if config['ANALYTICS_ENABLED'] -%}
    {{ analytics_macros.render_analytics_noscript(config['ANALYTICS_BASE_URL'], config['ANALYTICS_SITE_ID']) }}
    {%- endif -%}
    <a href="#main-container" class="is-sr-only">Skip to main content</a>
    {% block header %}
    {% include "base/header.html" %}
    {% endblock header %}
  </header>
  <main class="container" id="main-container">
    {% block alerts %}{{ macros.alerts(get_alerts()) }}{% endblock alerts %}
    {% block content %}
    {% endblock content %}
  </main>
  <footer>
    {% block footer %}
    {% include "base/footer.html" %}
    {% endblock footer %}
  </footer>
  </body>
</html>
