#  Created by tag_check.write_version
#  NEVER CHECK THIS version.py file INTO GIT.
#  It should be generated when the package is build for distribution.
__when__ = '16/03/2020 18:49:34'
__version__ = '0.0.1-dev1'
