@import url('https://fonts.googleapis.com/css?family=Open+Sans:400,400i,600,700')

// accessible color scheme added 01-27-2020:
$blue: #086db1
$blue-light: #1c8bd6
$blue-dark: #005e9d
$red: #cd0200
$red-light: #e30906
$red-dark: #b31b1b
$orange: #b55c06
$orange-light: #d47500
$orange-dark: #994d04
$yellow: #b18e0f
$yellow-light: #d9af14
$yellow-dark: #8d7109
$green: #008412
$green-light: #009917
$green-dark: #005909
// color associations:
$red-dark-darker: darken($red-dark, 20%)
$green-dark-darker: darken($green-dark, 4%)
$primary: $red_dark
$primary-invert: #fff
$warning: $orange
$warning-invert: #fff
$custom-colors: $red-dark-darker

// Variables, inspired by Spacelab theme https://jenil.github.io/bulmaswatch/spacelab/
$grey-darker: #2d2d2d
$grey-dark: #333
$grey: #757575
$grey-light: hsl(0, 0%, 60%)
$grey-lighter: hsl(0, 0%, 80%)

// $orange: #d47500
// $green: #3cb521
// $blue: #0068ac
// $red: #cd0200

$slate: #446e9b
$slate-invert: #fff

$cul-gray: #222
$cul-red: #b31b1b
$cul-red-darker: darken($cul-red, 15%)
$cul-red-darker-invert: #fff

// $primary: $cul-red
// $primary-invert: #fff
// $warning: $orange
// $warning-invert: #fff
// $custom-colors: $cul-red-darker

$subtitle-color: $grey

$navbar-item-active-color: $primary
$navbar-item-hover-background-color: transparent

$family-sans-serif: 'Open Sans', 'Lucida Grande', 'Helvetica Neue', Helvetica, Arial, sans-serif
$family-smalltext: 'Lucida Grande', 'Helvetica Neue', Helvetica, Arial, sans-serif

$size-7: .85rem
/* changed when base text size is 14px */

$box-padding: .75rem
$box-radius: 2px

$message-body-border-width: 1px
$message-header-body-border-width: 1px

$subtitle-color: $grey

$navbar-item-active-color: $primary
$navbar-item-hover-background-color: transparent

$notification-padding: .75rem 1.25rem

$button-hover-color: hsl(0, 0%, 50%)

@import 'bulma/bulma'
@import 'extensions/bulma-switch/switch'
@import '_overrides'
@import '_custom'
